package  
{
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;
	import org.flixel.FlxState;
	import org.flixel.FlxText;
	import org.flixel.FlxTilemap;
	import org.flixel.FlxU;
	/**
	 * Game State
	 * @author Basile Desloges
	 */
	public class GameState extends FlxState
	{
		private var player : Player;
		
		private var village : FlxSprite;
		[Embed(source='../assets/village.png')]
		private var imgVillage : Class;
		
		private var mapSize : Number;
		private var tilemap : FlxTilemap;
		[Embed(source = '../assets/tiles.png')]
		private var imgTiles : Class;
		
		private var water : FlxGroup;
		
		private var radios : FlxGroup;
		
		private var oasis : FlxSprite;
		[Embed(source='../assets/oasis.png')]
		private var imgOasis : Class;
		
		private var frameThirst : FlxSprite;
		private var insideThirst : FlxSprite;
		private var barThirst : FlxSprite;
		
		private var maxTime : Number;
		private var currentTime : Number;
		private var drinkTime : Number;
		private var scoreTime : Number;
		
		private var radioFounds : uint;
		private var maxRadios : uint;
		
		private var xRadioUI : Number;
		
		private var oasisFound : Boolean;
		
		private var score : Number;
		private var endPoints : Number;
		private var radioPoints : Number;
		private var oasisPoints : Number;
		
		private var endTextTitle : FlxText;
		private var endText : FlxText;
		
		override public function create():void {
			bgColor = 0xffb29b82;
			
			mapSize = 128;
			tilemap = initTilemap(mapSize);
			add(tilemap);
			
			oasis = new FlxSprite(0, 0, imgOasis);
			add(oasis);
			oasisFound = false;
			
			water = new FlxGroup();
			add(water);
			
			player = new Player(tilemap.width / 2, tilemap.height / 2);
			add(player);
			
			village = new FlxSprite(0, 0, imgVillage);
			add(village);
			
			Radio.reset();
			radios = new FlxGroup();
			add(radios);
			maxRadios = 5;
			radioFounds = 0;
			xRadioUI = FlxG.width;
			
			FlxG.follow(player);
			tilemap.follow(-1);
			initMap();
			
			drawThirstStatus();
			barThirst.scale.x = getPercentThirst();
			
			maxTime = 100;
			currentTime = maxTime;
			drinkTime = 10;
			scoreTime = 0;
			
			endPoints = 1000;
			radioPoints = 500;
			oasisPoints = 500;
			
			super.create();
		}
		
		override public function update():void {
			if( !player.dead ){
				// Endgame
				FlxU.overlap(player, village, overlapPlayerVillage);
				
				// Oasis
				FlxU.overlap(player, oasis, overlapPlayerOasis);
				
				// Radios
				FlxU.overlap(player, radios, overlapPlayerRadio);
				
				// Thirst
				FlxU.overlap(player, water, overlapPlayerWater);
				
				currentTime -= FlxG.elapsed;
				if ( currentTime < 0 ) {
					gameOver(scoreTime);
				}
				scoreTime += FlxG.elapsed;
			
				barThirst.scale.x = getPercentThirst();
			}
			
			//
			if ( FlxG.keys.justPressed("X") && player.dead ) {
				FlxG.state = new MenuState();
			}
			
			super.update();
			
			if ( player.x < 0 || player.x > tilemap.width - player.width ||
				 player.y < 0 || player.y > tilemap.height - player.height) {
				mapEnd();
			}
			
		}
		
		public function getPercentThirst() : Number {
			return currentTime / maxTime;
		}
		
		private function initTilemap(mapSize : uint) : FlxTilemap {
			var map : Array = new Array();
			var p : Number;
			
			for (var i:uint ; i < mapSize * mapSize ; ++i) {
				
				// Mise en place des tiles au hasard
				p = FlxU.random();
				if ( p > 0.9 ) {
					p = 0;
				}
				else if ( p > 0.8) {
					p = 4;
				}
				else if ( p > 0.7) {
					p = 3;
				}
				else if ( p > 0.6) {
					p = 2;
				}
				else if ( p > 0.5) {
					p = 1;
				}
				else {
					p = 0;
				}
				map[i] = p;
			}
			
			tilemap = new FlxTilemap();
			tilemap.drawIndex = 0;
			tilemap.loadMap(FlxTilemap.arrayToCSV(map, mapSize), imgTiles, 16, 16);
			
			return tilemap;
		}
		
		private function initMap() : void {
			for (var i : uint = 0 ; i < tilemap.widthInTiles ; ++i) {
				for (var j : uint = 0 ; j < tilemap.heightInTiles ; ++j) {
					if (FlxU.random() < 0.005) {
						water.add(new Water(i * (tilemap.width / tilemap.widthInTiles), j * (tilemap.height / tilemap.heightInTiles)));
					}
				}
			}
			
			// Village position
			// Up/Down or Left/Right
			if (FlxU.random() > 0.5) {
				//Up/Down
				if (FlxU.random() > 0.5) {
					//Up
					village.y = 0;
				}
				else {
					//Down
					village.y = tilemap.height - village.height;
				}
				
				village.x = Math.min(FlxU.random() * tilemap.width, tilemap.width - village.width);
			}
			else {
				//Left/Right
				if (FlxU.random() > 0.5) {
					//Left
					village.x = 0;
				}
				else {
					//Right
					village.x = tilemap.width - village.width;
				}
				
				village.y = Math.min(FlxU.random() * tilemap.height, tilemap.height - village.height);
			}
			
			// Radios positions
			var radiox : Number;
			var radioy : Number;
			trace("height : " + tilemap.height + ", width : " + tilemap.width);
			for (i = 0 ; i < maxRadios ; ++i) {
				trace("radio " + i);
				if (FlxU.random() > 0.5) {
					//Up/Down
					if (FlxU.random() > 0.5) {
						//Up
						radioy = 160 + FlxU.random() * 160;
						trace("\tUP, y : " + radioy);
					}
					else {
						//Down
						radioy = tilemap.height - 160 - FlxU.random() * 160;
						trace("\tDOWN, y : " + radioy);
					}
					
					radiox = 160 + FlxU.random() * (tilemap.width - 160);
					trace("\tx : " + radiox);
				}
				else {
					//Left/Right
					if (FlxU.random() > 0.5) {
						//Left
						radiox = 160 + FlxU.random() * 160;
						trace("\tLEFT, x : " + radiox);
					}
					else {
						//Right
						radiox = tilemap.width - 160 - FlxU.random() * 160;
						trace("\tRIGHT, x : " + radiox);
					}
					
					radioy = 160 + FlxU.random() * (tilemap.height - 160);
					trace("\ty : " + radioy);
				}
				trace("\tradio : " + radiox + "," + radioy );
				radios.add(new Radio(radiox, radioy));
			}
			
			// Oasis position
			if ( FlxU.random() < 0.5 ) {
				oasis.x = FlxU.random() * (tilemap.width / 2 - oasis.width - 320);	
			}
			else {
				oasis.x = tilemap.width / 2 + 320 + FlxU.random() * (tilemap.width / 2 - oasis.width - 320);
			}
			if ( FlxU.random() < 0.5 ) {
				oasis.y = FlxU.random() * (tilemap.height / 2 - oasis.height - 320);	
			}
			else {
				oasis.y = tilemap.height / 2 + 320 + FlxU.random() * (tilemap.height / 2 - oasis.height - 320);
			}
			
			trace("oasis : " + oasis.x + "," + oasis.y);
			
		}
	
		private function mapEnd() : void {
			Message.newMessage("Mmmh, it seems I'm going in the wrong direction, there's nothing over their...\nI should go back...", 5);
		}
		
		private function overlapPlayerWater(p : Player, w : Water) : void {
			drink();
			w.kill();
		}
		
		private function overlapPlayerVillage(p : Player, v : FlxSprite) : void {
			// Kill player
			player.kill();
			
			// Calculate score
			score = FlxU.floor(scoreTime * 100) / 100;
			score += endPoints;
			
			// Display message
			endTextTitle = new FlxText(0, FlxG.height / 2 - 70, FlxG.width, "YOU ARE SAVED");
			endTextTitle.setFormat(null, 32, MenuState.textColor, "center", MenuState.shadowColor);
			endTextTitle.scrollFactor.x = 0;
			endTextTitle.scrollFactor.y = 0;
			add(endTextTitle);
			
			endText = new FlxText(0, FlxG.height / 2, FlxG.width);
			endText.setFormat(null, 8, MenuState.shadowColor, "center");
			endText.text = "You found a village. The inhabitants give you some water, and some shelter. You are saved...\n";
			// txt perso suivant la fin
			if ( radioFounds == 0 ) {
				endText.text += "But you're still stuck in a desert... Maybe you'll have to go back in it to look for help.";
			}
			else if ( radioFounds == maxRadios) {
				endText.text += "Moreover, you found electronic pieces, enough to make a radio. You will be able to call for help !";
			}
			else {
				endText.text += "You found some pieces of radio in the desert. It's still in pieces, and you haven't got them all, but at least you know what you're looking for, and you will find them in the desert.";
			}
			endText.text += "\nBut not now... now, you can rest...\n\nFinal score : " + score + "\n Press X to go back to the menu";
			endText.scrollFactor.x = 0;
			endText.scrollFactor.y = 0;
			add(endText);
		}
		
		private function overlapPlayerRadio(p : Player, r : Radio) : void {
			var idx : uint = r.index;
			r.kill();
			++radioFounds;
			score += radioPoints;
			
			xRadioUI -= 12;
			var radioUI : Radio = new Radio(xRadioUI, FlxG.height - 12, idx);
			radioUI.scrollFactor.x = 0;
			radioUI.scrollFactor.y = 0;
			radioUI.origin.x = 0;
			radioUI.origin.y = 0;
			radioUI.scale.x = 0.5;
			radioUI.scale.y = 0.5;
			add(radioUI);
		}
		
		private function overlapPlayerOasis(p : Player, o : FlxSprite) : void {
			if (!oasisFound) {
				score += oasisPoints;
				maxTime *= 2;
			}
			currentTime = maxTime;
			oasisFound = true;
		}
		
		private function drawThirstStatus() : void {
			frameThirst = new FlxSprite(4, FlxG.height - 14);
			frameThirst.createGraphic(50, 10, 0xff000000);
			frameThirst.scrollFactor.x = 0;
			frameThirst.scrollFactor.y = 0;
			add(frameThirst);
			
			insideThirst = new FlxSprite(5, FlxG.height - 13);
			insideThirst.createGraphic(48, 8, 0xffe6e6ff);
			insideThirst.scrollFactor.x = 0;
			insideThirst.scrollFactor.y = 0;
			add(insideThirst);
			
			barThirst = new FlxSprite(5, FlxG.height - 13);
			barThirst.createGraphic(48, 8, 0xff8282ff);
			barThirst.origin.x = 0;
			barThirst.origin.y = 0;
			barThirst.scrollFactor.x = 0;
			barThirst.scrollFactor.y = 0;
			add(barThirst);
		}
		
		private function drink() : void {
			currentTime += drinkTime;
			if ( currentTime > maxTime ) {
				currentTime = maxTime;
			}
		}
		
		private function gameOver(time : Number ) : void {
			// Kill player
			player.kill();
			
			// calculate score and show game over text
			score = FlxU.floor(scoreTime * 100) / 100;
			endTextTitle = new FlxText(0, FlxG.height / 2 - 70, FlxG.width, "GAME OVER");
			endTextTitle.setFormat(null, 32, MenuState.textColor, "center", MenuState.shadowColor);
			endTextTitle.scrollFactor.x = 0;
			endTextTitle.scrollFactor.y = 0;
			add(endTextTitle);
			
			endText = new FlxText(0, FlxG.height / 2, FlxG.width);
			endText.setFormat(null, 8, MenuState.shadowColor, "center");
			endText.text = "Final score : " + score + "\n Press X to go back to the menu";
			endText.scrollFactor.x = 0;
			endText.scrollFactor.y = 0;
			add(endText);
		}
		
	}

}